@@ -60,6 +60,7 @@ static void setdberrmsg(TCLIST *emsgs, TCTDB *tdb, const char *msg);
 static void setarthtml(TCMPOOL *mpool, TCMAP *cols, int64_t id, int bhl, bool tiny);
 static TCLIST *searcharts(TCMPOOL *mpool, TCTDB *tdb, const char *cond, const char *expr,
                           const char *order, int max, int skip, bool ls);
+static void getdaterange(const char *expr, int64_t *lowerp, int64_t *upper);
 static bool putfile(TCMPOOL *mpool, const char *path, const char *name,
                     const char *ptr, int size);
 static bool outfile(TCMPOOL *mpool, const char *path);
@@ -949,6 +950,45 @@ static void dosession(TCMPOOL *mpool){
       if(tcmapget2(vars, "prev") || tcmapget2(vars, "next")) tcmapput2(vars, "page", "true");
       tcmapputlist(vars, "arts", arts);
     }
+    if(*p_cond == '\0'){
+      res = tcmpoollistnew(mpool);
+      char numbuf[NUMBUFSIZ];
+      tcdatestrwww(now, INT_MAX, numbuf);
+      int year = tcatoi(numbuf);
+      int minyear = year;
+      res = searcharts(mpool, tdb, "cdate", "x", "_cdate", 1, 0, true);
+      if(tclistnum(res) > 0){
+        int64_t id = tcatoi(tclistval2(res, 0));
+        TCMAP *cols = tcmpoolpushmap(mpool, id > 0 ? dbgetart(tdb, id) : NULL);
+        const char *value = cols ? tcmapget2(cols, "cdate") : NULL;
+        if(value){
+          int64_t cdate = tcstrmktime(value);
+          tcdatestrwww(cdate, INT_MAX, numbuf);
+          minyear = tcatoi(numbuf);
+        }
+      }
+      TCLIST *arcyears = tcmpoollistnew(mpool);
+      for(int i = 0; i < 100 && year >= minyear; i++){
+        sprintf(numbuf, "%04d", year);
+        res = searcharts(mpool, tdb, "cdate", numbuf, "cdate", 1, 0, true);
+        if(tclistnum(res) > 0){
+          TCMAP *arcmonths = tcmpoolpushmap(mpool, tcmapnew2(TINYBNUM));
+          for(int month = 0; month <= 12; month++){
+            sprintf(numbuf, "%04d-%02d", year, month);
+            res = searcharts(mpool, tdb, "cdate", numbuf, "cdate", 1, 0, true);
+            rnum = tclistnum(res);
+            sprintf(numbuf, "%02d", month);
+            if(rnum > 0) tcmapprintf(arcmonths, numbuf, "%d", rnum);
+          }
+          if(tcmaprnum(arcmonths) > 0){
+            tcmapprintf(arcmonths, "year", "%d", year);
+            tclistpushmap(arcyears, arcmonths);
+          }
+        }
+        year--;
+      }
+      if(tclistnum(arcyears) > 0) tcmapputlist(vars, "arcyears", arcyears);
+    }
   } else if(*g_frontpage != '\0' && strcmp(p_act, "timeline")){
     // front view
     if(!auth){
@@ -1287,6 +1327,30 @@ static TCLIST *searcharts(TCMPOOL *mpool, TCTDB *tdb, const char *cond, const ch
     TDBQRY *qry = tcmpoolpush(mpool, tctdbqrynew(tdb), (void (*)(void *))tctdbqrydel);
     tctdbqryaddcond(qry, "name", TDBQCFTSEX, expr);
     qrys[qnum++] = qry;
+  } else if(!strcmp(cond, "cdate")){
+    int64_t lower, upper;
+    getdaterange(expr, &lower, &upper);
+    char numbuf[NUMBUFSIZ*2];
+    sprintf(numbuf, "%lld,%lld", (long long)lower, (long long)upper);
+    TDBQRY *qry = tcmpoolpush(mpool, tctdbqrynew(tdb), (void (*)(void *))tctdbqrydel);
+    tctdbqryaddcond(qry, "cdate", TDBQCNUMBT, numbuf);
+    qrys[qnum++] = qry;
+  } else if(!strcmp(cond, "mdate")){
+    int64_t lower, upper;
+    getdaterange(expr, &lower, &upper);
+    char numbuf[NUMBUFSIZ*2];
+    sprintf(numbuf, "%lld,%lld", (long long)lower, (long long)upper);
+    TDBQRY *qry = tcmpoolpush(mpool, tctdbqrynew(tdb), (void (*)(void *))tctdbqrydel);
+    tctdbqryaddcond(qry, "mdate", TDBQCNUMBT, numbuf);
+    qrys[qnum++] = qry;
+  } else if(!strcmp(cond, "xdate")){
+    int64_t lower, upper;
+    getdaterange(expr, &lower, &upper);
+    char numbuf[NUMBUFSIZ*2];
+    sprintf(numbuf, "%lld,%lld", (long long)lower, (long long)upper);
+    TDBQRY *qry = tcmpoolpush(mpool, tctdbqrynew(tdb), (void (*)(void *))tctdbqrydel);
+    tctdbqryaddcond(qry, "xdate", TDBQCNUMBT, numbuf);
+    qrys[qnum++] = qry;
   } else if(!strcmp(cond, "owner")){
     TDBQRY *qry = tcmpoolpush(mpool, tctdbqrynew(tdb), (void (*)(void *))tctdbqrydel);
     tctdbqryaddcond(qry, "owner", TDBQCSTREQ, expr);
@@ -1366,6 +1430,60 @@ static TCLIST *searcharts(TCMPOOL *mpool, TCTDB *tdb, const char *cond, const ch
 }
 
 
+/* get the range of a date expression */
+static void getdaterange(const char *expr, int64_t *lowerp, int64_t *upperp){
+  while(*expr == ' '){
+    expr++;
+  }
+  unsigned int year = 0;
+  for(int i = 0; i < 4 && *expr >= '0' && *expr <= '9'; i++){
+    year = year * 10 + *expr - '0';
+    expr++;
+  }
+  if(*expr == '-' || *expr == '/') expr++;
+  unsigned int month = 0;
+  for(int i = 0; i < 2 && *expr >= '0' && *expr <= '9'; i++){
+    month = month * 10 + *expr - '0';
+    expr++;
+  }
+  if(*expr == '-' || *expr == '/') expr++;
+  unsigned int day = 0;
+  for(int i = 0; i < 2 && *expr >= '0' && *expr <= '9'; i++){
+    day = day * 10 + *expr - '0';
+    expr++;
+  }
+  int lag = tcjetlag() / 3600;
+  int64_t lower, upper;
+  char numbuf[NUMBUFSIZ*2];
+  if(day > 0){
+    sprintf(numbuf, "%04u-%02u-%02uT00:00:00%+03d:00", year, month, day, lag);
+    lower = tcstrmktime(numbuf);
+    upper = lower + 60 * 60 * 24 - 1;
+  } else if(month > 0){
+    sprintf(numbuf, "%04u-%02u-01T00:00:00%+03d:00", year, month, lag);
+    lower = tcstrmktime(numbuf);
+    month++;
+    if(month > 12){
+      year++;
+      month = 1;
+    }
+    sprintf(numbuf, "%04u-%02u-01T00:00:00%+03d:00", year, month, lag);
+    upper = tcstrmktime(numbuf) - 1;
+  } else if(year > 0){
+    sprintf(numbuf, "%04u-01-01T00:00:00%+03d:00", year, lag);
+    lower = tcstrmktime(numbuf);
+    year++;
+    sprintf(numbuf, "%04u-01-01T00:00:00%+03d:00", year, lag);
+    upper = tcstrmktime(numbuf) - 1;
+  } else {
+    lower = INT64_MIN / 2;
+    upper = INT64_MAX / 2;
+  }
+  *lowerp = lower;
+  *upperp = upper;
+}
+
+
 /* store a file */
 static bool putfile(TCMPOOL *mpool, const char *path, const char *name,
                     const char *ptr, int size){
